local next = next
local nilCandleInserterTable = {inserterEntity = false, inserterType = "", inserterGrid = {}, playerIndex = false}	--unit_number?
local nilCandleCacheTable = {inserterGrid = {isReady = false}}
local inserterTypes = { 
	["candle-basic-inserter"] = "normal",
	["candle-fast-inserter"] = "normal",
	["candle-filter-inserter"] = "normal",
	["candle-stack-inserter"] = "normal",
	["candle-stack-filter-inserter"] = "normal",
	["candle-basic-long-inserter"] = "long",
	["candle-fast-long-inserter"] = "long",
	["candle-filter-long-inserter"] = "long",
	["candle-stack-long-inserter"] = "long",
	["candle-stack-filter-long-inserter"] = "long",
	["candle-advanced-inserter"] = "advanced",
	["candle-advanced-stack-inserter"] = "advanced"
}
local gridTables = { 
	advanced = { true, true, true, true, true, true, true, true, true, true, true, true, false, true, true, true, true, true, true, true, true, true, true, true, true },
	long = { true, true, true, true, true, true, false, false, false, true, true, false, false, false, true, true, false, false, false, true, true, true, true, true, true },
	normal = { false, false, false, false, false, false, true, true, true, false, false, true, false, true, false, false, true, true, true, false, false, false, false, false, false }
}
local pickupTable = { {-2, -2}, {-1, -2}, {0, -2}, {1, -2}, {2, -2}, {-2, -1}, {-1, -1}, {0, -1}, {1, -1}, {2, -1}, {-2, 0}, {-1, 0}, {0, 0}, {1, 0}, {2, 0}, {-2, 1}, {-1, 1}, {0, 1}, {1, 1}, {2, 1}, {-2, 2}, {-1, 2}, {0, 2}, {1, 2}, {2, 2} }
local dropTables = { 
	near = { {-1.8, -1.8}, {-0.8, -1.8}, {0, -1.8}, {0.8, -1.8}, {1.8, -1.8}, {-1.8, -0.8}, {-0.8, -0.8}, {0, -0.8}, {0.8, -0.8}, {1.8, -0.8}, {-1.8, 0}, {-0.8, 0}, {0, 0}, {0.8, 0}, {1.8, 0}, {-1.8, 0.8}, {-0.8, 0.8}, {0, 0.8}, {0.8, 0.8}, {1.8, 0.8}, {-1.8, 1.8}, {-0.8, 1.8}, {0, 1.8}, {0.8, 1.8}, {1.8, 1.8} },
	far = { {-2.2, -2.2}, {-1.2, -2.2}, {0, -2.2}, {1.2, -2.2}, {2.2, -2.2}, {-2.2, -1.2}, {-1.2, -1.2}, {0, -1.2}, {1.2, -1.2}, {2.2, -1.2}, {-2.2, 0}, {-1.2, 0}, {0, 0}, {1.2, 0}, {2.2, 0}, {-2.2, 1.2}, {-1.2, 1.2}, {0, 1.2}, {1.2, 1.2}, {2.2, 1.2}, {-2.2, 2.2}, {-1.2, 2.2}, {0, 2.2}, {1.2, 2.2}, {2.2, 2.2} }
}

--[[ HELPER FUNCTIONS ]]--
local function reconfigureInserterGrid(parent, gridTableName, inserterType)
	local gridTable = gridTables[inserterType]
	local newInserterGrid = parent.add({ type = "table", name = gridTableName, colspan = 5, style = "lyr_gridTable" })
	
	for gridNumber, gridType in next, gridTable do
		if gridNumber == 13 then
			newInserterGrid.add({ type = "checkbox", name = "invalidGrid_"..gridNumber, state = false, style = "lyr_baseGrid" })
		elseif gridType then
			newInserterGrid.add({ type = "checkbox", name = "inserterGrid_"..gridNumber, state = false, style = "lyr_validGrid" })
		else
			newInserterGrid.add({ type = "checkbox", name = "invalidGrid_"..gridNumber, state = false, style = "lyr_invalidGrid" })
		end
	end
	
	return newInserterGrid
end

local function programmingInterface(playerIndex)
	local inserterTable = global.candleInserterTable[playerIndex]
	local player, inserterType, inserterGrid = game.players[playerIndex], inserterTable.inserterType, inserterTable.inserterGrid
	if player.gui.center.candleMainFrame then player.gui.center.candleMainFrame.destroy() end
	
	local mainFrame = player.gui.center.add({ type = "frame", name = "candleMainFrame", direction = "vertical" })
	mainFrame.add({ type = "label", name = "caption", caption = "K&L Interface", style = "frame_caption_label_style" })
	local mainTable = mainFrame.add({ type = "table", name = "mainTable", colspan = 2, style = "lyr_table" })
		mainTable.add({ type = "label", name = "pickupGridHeader", caption = "Pickup Grid" })
		mainTable.add({ type = "label", name = "dropGridHeader", caption = "Drop Grid" })
			local pickupGridTable = reconfigureInserterGrid(mainTable, "pickupGridTable", inserterType)
			local dropGridTable = reconfigureInserterGrid(mainTable, "dropGridTable", inserterType)
		local buttonGridTable = mainTable.add({ type = "table", name = "buttonGridTable", colspan = 2, style = "lyr_gridTable" })
			buttonGridTable.add({ type = "button", name = "candleImport", caption = "Import", style = "lyr_smallButton" })
			buttonGridTable.add({ type = "button", name = "candleExport", caption = "Export", style = "lyr_smallButton" })
		local candleNear = mainTable.add({ type = "checkbox", name = "candleNear", state = false, caption = "Drop Near", style = "checkbox_style" })
		mainTable.add({ type = "button", name = "candleAccept", caption = "Accept", style = "lyr_button" })
		mainTable.add({ type = "button", name = "candleCancel", caption = "Close", style = "lyr_button" })
	
	if inserterGrid and next(inserterGrid) then
		pickupGridTable["inserterGrid_"..inserterGrid.pickupGrid].state = true
		dropGridTable["inserterGrid_"..inserterGrid.dropGrid].state = true
		candleNear.state = inserterGrid.dropType == "near"
	end
end

local function getProperties(playerIndex, inserterEntity, itemName, isGhost)
	local inserterName, inserterDirection = isGhost and inserterEntity.ghost_name or inserterEntity.name, inserterEntity.direction
	local inserterGrid, inserterType, itemType = {}, inserterTypes[inserterName], inserterTypes[itemName]
	
	global.candleInserterTable[playerIndex] = nilCandleInserterTable
	
	if inserterEntity.pickup_position and inserterEntity.drop_position then 
		local inserterPosition = inserterEntity.position
		local pickupGrid, dropGrid, dropType = 0, 0, "far"
		local distanceTables = { 
			pickup = { x = inserterPosition.x - inserterEntity.pickup_position.x, y = inserterPosition.y - inserterEntity.pickup_position.y },
			drop = { x = inserterPosition.x - inserterEntity.drop_position.x, y = inserterPosition.y - inserterEntity.drop_position.y }
		}
		
		for distanceOf, distanceTable in next, distanceTables do
			for distanceAxis, distance in next, distanceTable do
				local base, modifier, multiplier = 0, 0, 1
				
				if distanceAxis == "y" then
					modifier = -1
					multiplier = 5
				end
				
				if distanceOf == "drop" then 
					if 		distance == 0		then		base = 3
					elseif 	distance > 2.175	then		base = 1
					elseif 	distance > 1.75	then		base = 1; dropType = "near"
					elseif 	distance > 1.175	then		base = 2
					elseif 	distance > 0.75	then		base = 2; dropType = "near"
					elseif 	distance < -2.175	then		base = 5
					elseif 	distance < -1.75	then		base = 5; dropType = "near"
					elseif 	distance < -1.175	then		base = 4
					elseif 	distance < -0.75	then		base = 4; dropType = "near"
					end
					
					dropGrid = dropGrid + (base + modifier) * multiplier
				else
					if 		distance == 0		then		base = 3
					elseif 	distance > 1.75	then		base = 1
					elseif 	distance > 0.75	then		base = 2
					elseif 	distance < -1.75	then		base = 5
					elseif 	distance < -0.75	then		base = 4
					end
					
					pickupGrid = pickupGrid + (base + modifier) * multiplier
				end
			end
		end
		
		if pickupGrid ~= 13 and dropGrid ~= 13 and pickupGrid ~= dropGrid then inserterGrid = {pickupGrid = pickupGrid, dropGrid = dropGrid, dropType = dropType} end
	end; if itemName and inserterGrid and next(inserterGrid) and not (gridTables[itemType][inserterGrid.pickupGrid] and gridTables[itemType][inserterGrid.dropGrid]) then inserterGrid = {} end
	
	global.candleInserterTable[playerIndex] = {inserterEntity = inserterEntity, inserterType = inserterTypes[inserterName], inserterGrid = inserterGrid, playerIndex = playerIndex}
end

local function reconfigureInserter(playerIndex)
	local inserterTable = global.candleInserterTable[playerIndex]
	local inserterEntity, inserterGrid = inserterTable.inserterEntity, inserterTable.inserterGrid
		if not inserterEntity.valid then global.candleInserterTable[playerIndex] = nilCandleInserterTable; return end
	local inserterPosition = inserterEntity.position
	local pickupDistances, dropDistances = pickupTable[inserterGrid.pickupGrid], dropTables[inserterGrid.dropType][inserterGrid.dropGrid]
	
	inserterEntity.pickup_position = {inserterPosition.x + pickupDistances[1], inserterPosition.y + pickupDistances[2]}
	inserterEntity.drop_position = {inserterPosition.x + dropDistances[1], inserterPosition.y + dropDistances[2]}
	
	global.candleInserterTable[playerIndex] = nilCandleInserterTable
end

--[[ EVENT HANDLERS ]]--
local onBuiltEntityFilter = {candle = true}
local function onBuiltEntity(event)
	local inserterEntity = event.created_entity
	local inserterName = inserterEntity.name
		if not onBuiltEntityFilter[inserterName:sub(1,6)] then return end
	local playerIndex = event.player_index
	local inserterGrid = global.candleInserterTable[playerIndex].inserterGrid
	
	inserterEntity.direction = 0
	global.candleInserterTable[playerIndex] = {inserterEntity = inserterEntity, inserterType = inserterTypes[inserterName], inserterGrid = inserterGrid, playerIndex = playerIndex}
	
	if inserterGrid and next(inserterGrid) then
		reconfigureInserter(playerIndex)
	else
		programmingInterface(playerIndex) 
	end
end

local onPutItemFilter = {candle = true}
local function onPutItem(event) 
	local playerIndex = event.player_index
	local player = game.players[playerIndex]
	local item = player.cursor_stack
		if not item.valid_for_read or not onPutItemFilter[item.name:sub(1,6)] then return end 
	local positionX, positionY, area = event.position.x, event.position.y; area = {{positionX-0.35, positionY-0.35}, {positionX+0.35, positionY+0.35}}
	local itemName, inserterEntity, entitySearch, isGhost = item.name
	if not inserterEntity then entitySearch = player.surface.find_entities_filtered({area = area, type = "entity-ghost"}); if entitySearch then for _, entity in next, entitySearch do if entity.ghost_type == "inserter" then inserterEntity = entity; isGhost = true end end end end
	if not inserterEntity then entitySearch = player.surface.find_entities_filtered({area = area, type = "inserter"}); if entitySearch then for _, entity in next, entitySearch do inserterEntity = entity end end end
	if not inserterEntity then global.candleInserterTable[playerIndex] = nilCandleInserterTable; return end;
	
	getProperties(playerIndex, inserterEntity, itemName, isGhost)
end

local onRotatedEntityFilter = {candle = true}
local function onRotatedEntity(event) 
	local inserterEntity = event.entity
		if not onRotatedEntityFilter[inserterEntity.name:sub(1,6)] then return end
	local inserterDirection = inserterEntity.direction
	local playerIndex = event.player_index
	
	if inserterDirection ~= 2 then
		inserterEntity.direction = (inserterDirection - 2)%8
		local pickupPosition, dropPosition = inserterEntity.pickup_position, inserterEntity.drop_position
		
		inserterEntity.direction = 0
		inserterEntity.pickup_position, inserterEntity.drop_position = pickupPosition, dropPosition
	else
		inserterEntity.direction = 0
	end
	
	getProperties(playerIndex, inserterEntity)
	programmingInterface(playerIndex)
end

local onGUIClickFilter = {inserterGrid = true, candleAccept = true, candleCancel = true, candleImport = true, candleExport = true}
local function onGUIClick(event)
	local element = event.element
	local elementName = element.name
		if not onGUIClickFilter[elementName:sub(1, 12)] then return end
	local playerIndex = event.player_index
	
	if elementName:sub(1, 12) == "inserterGrid" then
		local parent = element.parent
		local active = tonumber(elementName:sub(14))
		
		for gridNumber = 1, 25 do
			if gridNumber ~= active and parent["inserterGrid_"..gridNumber] then
				parent["inserterGrid_"..gridNumber].state = false
			end
		end
	else 
		local mainFrame = game.players[playerIndex].gui.center.candleMainFrame
		local mainTable = mainFrame.mainTable
		local pickupGridTable = mainTable.pickupGridTable
		local dropGridTable = mainTable.dropGridTable
		
		if elementName == "candleCancel" then
			global.candleInserterTable[playerIndex] = nilCandleInserterTable
			mainFrame.destroy()
		elseif elementName == "candleImport" then
			local cachedInserterGrid = global.candleCacheTable[playerIndex].inserterGrid
			
			if cachedInserterGrid.isReady and pickupGridTable["inserterGrid_"..cachedInserterGrid.pickupGrid] and dropGridTable["inserterGrid_"..cachedInserterGrid.dropGrid] then
				global.candleInserterTable[playerIndex].inserterGrid = cachedInserterGrid
				reconfigureInserter(playerIndex)
				mainFrame.destroy()
			end
		else
			local pickupGrid, dropGrid, dropType
			
			for gridNumber = 1, 25 do
				if pickupGridTable["inserterGrid_"..gridNumber] and dropGridTable["inserterGrid_"..gridNumber] then
					if pickupGridTable["inserterGrid_"..gridNumber].state then
						pickupGrid = gridNumber
					end
					if dropGridTable["inserterGrid_"..gridNumber].state then
						dropGrid = gridNumber
					end
				end
			end; dropType = mainTable.candleNear.state and "near" or "far"
			
			if pickupGrid ~= nil and dropGrid ~= nil and pickupGrid ~= dropGrid then
				if elementName == "candleExport" then
					global.candleCacheTable[playerIndex].inserterGrid = {pickupGrid = pickupGrid, dropGrid = dropGrid, dropType = dropType, isReady = true}
				end; global.candleInserterTable[playerIndex].inserterGrid = {pickupGrid = pickupGrid, dropGrid = dropGrid, dropType = dropType}
				
				reconfigureInserter(playerIndex)
				mainFrame.destroy()
			end
		end
	end
end

local function onPlayerJoined(event)
	local playerIndex = event.player_index
	
	global.candleInserterTable[playerIndex] = nilCandleInserterTable
	global.candleCacheTable[playerIndex] = nilCandleCacheTable
end

local function onPlayerLeft(event)
	local playerIndex = event.player_index
	
	global.candleInserterTable[playerIndex] = nil
	global.candleCacheTable[playerIndex] = nil
end

local function onConfigurationChanged()
	if not global.candleInserterTable then global.candleInserterTable = {} end
	if not global.candleCacheTable then global.candleCacheTable = {} end
	
	if global.candleTable then
		for playerIndex, _ in next, global.candleTable do 
			global.candleInserterTable[playerIndex] = nilCandleInserterTable
			global.candleCacheTable[playerIndex] = nilCandleCacheTable
		end
		
		global.candleTable = nil
	end
end

local function onInitialize()
	if not global.candleInserterTable then global.candleInserterTable = {} end
	if not global.candleCacheTable then global.candleCacheTable = {} end
end

--[[ EVENT REGISTRATION ]]--
script.on_init(onInitialize)
script.on_configuration_changed(onConfigurationChanged)
script.on_event(defines.events.on_player_left_game, onPlayerLeft)
script.on_event(defines.events.on_player_joined_game, onPlayerJoined)
script.on_event(defines.events.on_gui_click, onGUIClick)
script.on_event(defines.events.on_player_rotated_entity, onRotatedEntity)
script.on_event(defines.events.on_put_item, onPutItem)
script.on_event(defines.events.on_built_entity, onBuiltEntity)