local next = next

local basePrototypes = {
	inserter = {
		type = "inserter", 
		flags = {"placeable-neutral", "placeable-player", "player-creation"}, 
		max_health = 40, 
		corpse = "small-remnants", 
		resistances = {{type = "fire", percent = 90}}, 
		vehicle_impact_sound ={filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65}, 
		working_sound = {match_progress_to_activity = true, 
			sound = {
				{filename = "__base__/sound/inserter-fast-1.ogg", volume = 0.75}, 
				{filename = "__base__/sound/inserter-fast-2.ogg", volume = 0.75}, 
				{filename = "__base__/sound/inserter-fast-3.ogg", volume = 0.75}, 
				{filename = "__base__/sound/inserter-fast-4.ogg", volume = 0.75}, 
				{filename = "__base__/sound/inserter-fast-5.ogg", volume = 0.75}
			}
		},
		collision_box = {{-0.15, -0.15}, {0.15, 0.15}}, 
		selection_box = {{-0.4, -0.35}, {0.4, 0.45}}, 
		pickup_position = {0, 0}, 
		insert_position = {0, 0}, 
		hand_base_shadow = {filename = "__base__/graphics/entity/burner-inserter/burner-inserter-hand-base-shadow.png", priority = "extra-high", width = 8, height = 34}, 
		hand_closed_shadow = {filename = "__base__/graphics/entity/burner-inserter/burner-inserter-hand-closed-shadow.png", priority = "extra-high", width = 18, height = 41}, 
		hand_open_shadow = {filename = "__base__/graphics/entity/burner-inserter/burner-inserter-hand-open-shadow.png", priority = "extra-high", width = 18, height = 41}, 
		circuit_wire_connection_point = inserter_circuit_wire_connection_point, 
		circuit_connector_sprites = inserter_circuit_connector_sprites, 
		circuit_wire_max_distance = inserter_circuit_wire_max_distance
	},
	item =  {
		type = "item", 
		flags = {"goes-to-quickbar"}, 
		subgroup = "inserter", 
		stack_size = 50
	},
	recipe = {
		type = "recipe",
		requester_paste_multiplier = 4
	}
}

local candlePrototypes = {
	["candle-basic-inserter"] = {
		inserter = {
			name = "candle-basic-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-basic-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-basic-inserter"}, 
			energy_per_movement = 5000, 
			energy_per_rotation = 5000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.03, 
			rotation_speed = 0.014, 
			fast_replaceable_group = "inserter", 
			hand_base_picture = {filename = "__base__/graphics/entity/inserter/inserter-hand-base.png", priority = "extra-high", width = 8, height = 33}, 
			hand_closed_picture = {filename = "__base__/graphics/entity/inserter/inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__base__/graphics/entity/inserter/inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__base__/graphics/entity/inserter/inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
		},
		item =  {
			name = "candle-basic-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-basic-inserter.png", 
			order = "x-c-a", 
			place_result = "candle-basic-inserter", 
		},
		recipe = {
			name = "candle-basic-inserter", 
			ingredients = {{"inserter", 1}}, 
			result = "candle-basic-inserter", 
			enabled = true, 
		}
	},
	["candle-fast-inserter"] = {
		inserter = {
			name = "candle-fast-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-fast-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-fast-inserter"}, 
			energy_per_movement = 7000, 
			energy_per_rotation = 7000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.07, 
			rotation_speed = 0.04, 
			fast_replaceable_group = "inserter", 
			hand_base_picture = {filename = "__base__/graphics/entity/fast-inserter/fast-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__base__/graphics/entity/fast-inserter/fast-inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__base__/graphics/entity/fast-inserter/fast-inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__base__/graphics/entity/fast-inserter/fast-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
		},
		item =  {
			name = "candle-fast-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-fast-inserter.png", 
			order = "x-c-c", 
			place_result = "candle-fast-inserter", 
		},
		recipe = {
			name = "candle-fast-inserter", 
			ingredients = {{"fast-inserter", 1}}, 
			result = "candle-fast-inserter", 
			enabled = false, 
		}
	},
	["candle-filter-inserter"] = {
		inserter = {
			name = "candle-filter-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-filter-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-filter-inserter"}, 
			energy_per_movement = 8000, 
			energy_per_rotation = 8000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.07, 
			rotation_speed = 0.04, 
			fast_replaceable_group = "inserter", 
			hand_base_picture = {filename = "__base__/graphics/entity/filter-inserter/filter-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__base__/graphics/entity/filter-inserter/filter-inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__base__/graphics/entity/filter-inserter/filter-inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__base__/graphics/entity/filter-inserter/filter-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
			filter_count = 5, 
		},
		item =  {
			name = "candle-filter-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-filter-inserter.png", 
			order = "x-c-e", 
			place_result = "candle-filter-inserter", 
		},
		recipe = {
			name = "candle-filter-inserter", 
			ingredients = {{"filter-inserter", 1}}, 
			result = "candle-filter-inserter", 
			enabled = false, 
		}
	},
	["candle-stack-inserter"] = {
		inserter = {
			name = "candle-stack-inserter",
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-stack-inserter"},
			energy_per_movement = 20000,
			energy_per_rotation = 20000,
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "1kW"},
			extension_speed = 0.07,
			rotation_speed = 0.04,
			fast_replaceable_group = "inserter",
			hand_base_picture = {filename = "__base__/graphics/entity/stack-inserter/stack-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34},
			hand_closed_picture = {filename = "__base__/graphics/entity/stack-inserter/stack-inserter-hand-closed.png", priority = "extra-high", width = 24, height = 41},
			hand_open_picture = {filename = "__base__/graphics/entity/stack-inserter/stack-inserter-hand-open.png", priority = "extra-high", width = 32, height = 41},
			platform_picture = {sheet = {filename = "__base__/graphics/entity/stack-inserter/stack-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}},
			stack = true, 
		},
		item =  {
			name = "candle-stack-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-inserter.png", 
			order = "x-c-g", 
			place_result = "candle-stack-inserter", 
		},
		recipe = {
			name = "candle-stack-inserter", 
			ingredients = {{"stack-inserter", 1}}, 
			result = "candle-stack-inserter", 
			enabled = false, 
		}
	}, 
	["candle-stack-filter-inserter"] = {
		inserter = {
			name = "candle-stack-filter-inserter",
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-filter-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-stack-filter-inserter"},
			energy_per_movement = 20000,
			energy_per_rotation = 20000,
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "1kW"},
			extension_speed = 0.07,
			rotation_speed = 0.04,
			fast_replaceable_group = "inserter",
			hand_base_picture = {filename = "__base__/graphics/entity/stack-filter-inserter/stack-filter-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34},
			hand_closed_picture = {filename = "__base__/graphics/entity/stack-filter-inserter/stack-filter-inserter-hand-closed.png", priority = "extra-high", width = 24, height = 41},
			hand_open_picture = {filename = "__base__/graphics/entity/stack-filter-inserter/stack-filter-inserter-hand-open.png", priority = "extra-high", width = 32, height = 41},
			platform_picture = {sheet = {filename = "__base__/graphics/entity/stack-filter-inserter/stack-filter-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}},
			filter_count = 1,
			stack = true, 
		},
		item =  {
			name = "candle-stack-filter-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-filter-inserter.png", 
			order = "x-c-i", 
			place_result = "candle-stack-filter-inserter", 
		},
		recipe = {
			name = "candle-stack-filter-inserter", 
			ingredients = {{"stack-filter-inserter", 1}}, 
			result = "candle-stack-filter-inserter", 
			enabled = false, 
		}
	}, 
	["candle-basic-long-inserter"] = {
		inserter = {
			name = "candle-basic-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-basic-long-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-basic-long-inserter"}, 
			energy_per_movement = 5000, 
			energy_per_rotation = 5000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.03, --0.0457
			rotation_speed = 0.014, --0.02
			fast_replaceable_group = "long-handed-inserter", 
			hand_size = 1.5, 
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-basic-long-inserter/candle-basic-long-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-basic-long-inserter/candle-basic-long-inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-basic-long-inserter/candle-basic-long-inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-basic-long-inserter/candle-basic-long-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
		},
		item =  {
			name = "candle-basic-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-basic-long-inserter.png", 
			order = "x-c-b", 
			place_result = "candle-basic-long-inserter", 
		},
		recipe = {
			name = "candle-basic-long-inserter", 
			ingredients = {{"long-handed-inserter", 1}}, 
			result = "candle-basic-long-inserter", 
			enabled = false, 
		}
	},
	["candle-fast-long-inserter"] = {
		inserter = {
			name = "candle-fast-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-fast-long-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-fast-long-inserter"}, 
			energy_per_movement = 7000, 
			energy_per_rotation = 7000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.07, 
			rotation_speed = 0.04, 
			fast_replaceable_group = "long-handed-inserter", 
			hand_size = 1.5, 
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-fast-long-inserter/candle-fast-long-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-fast-long-inserter/candle-fast-long-inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-fast-long-inserter/candle-fast-long-inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-fast-long-inserter/candle-fast-long-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
		},
		item =  {
			name = "candle-fast-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-fast-long-inserter.png", 
			order = "x-c-d", 
			place_result = "candle-fast-long-inserter", 
		},
		recipe = {
			name = "candle-fast-long-inserter", 
			ingredients = {{"fast-inserter", 1}, {"iron-gear-wheel", 1}, {"iron-plate", 1}}, 
			result = "candle-fast-long-inserter", 
			enabled = false, 
		}
	},
	["candle-filter-long-inserter"] = {
		inserter = {
			name = "candle-filter-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-filter-long-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-filter-long-inserter"}, 
			energy_per_movement = 8000, 
			energy_per_rotation = 8000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.07, 
			rotation_speed = 0.04, 
			fast_replaceable_group = "long-handed-inserter", 
			hand_size = 1.5, 
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-filter-long-inserter/candle-filter-long-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-filter-long-inserter/candle-filter-long-inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-filter-long-inserter/candle-filter-long-inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-filter-long-inserter/candle-filter-long-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
			filter_count = 5, 
		},
		item =  {
			name = "candle-filter-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-filter-long-inserter.png", 
			order = "x-c-f", 
			place_result = "candle-filter-long-inserter", 
		},
		recipe = {
			name = "candle-filter-long-inserter", 
			ingredients = {{"filter-inserter", 1}, {"iron-gear-wheel", 1}, {"iron-plate", 1}}, 
			result = "candle-filter-long-inserter", 
			enabled = false, 
		}
	},
	["candle-stack-long-inserter"] = {
		inserter = {
			name = "candle-stack-long-inserter",
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-long-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-stack-long-inserter"},
			energy_per_movement = 20000,
			energy_per_rotation = 20000,
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "1kW"},
			extension_speed = 0.07,
			rotation_speed = 0.04,
			fast_replaceable_group = "inserter",
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-long-inserter/candle-stack-long-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34},
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-long-inserter/candle-stack-long-inserter-hand-closed.png", priority = "extra-high", width = 24, height = 41},
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-long-inserter/candle-stack-long-inserter-hand-open.png", priority = "extra-high", width = 32, height = 41},
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-long-inserter/candle-stack-long-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}},
			stack = true, 
		},
		item =  {
			name = "candle-stack-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-long-inserter.png", 
			order = "x-c-h", 
			place_result = "candle-stack-long-inserter" 
		},
		recipe = {
			name = "candle-stack-long-inserter", 
			ingredients = {{"stack-inserter", 1}, {"iron-gear-wheel", 1}, {"iron-plate", 1}}, 
			result = "candle-stack-long-inserter", 
			enabled = false
		}
	}, 
	["candle-stack-filter-long-inserter"] = {
		inserter = {
			name = "candle-stack-filter-long-inserter",
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-filter-long-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-stack-filter-long-inserter"},
			energy_per_movement = 20000,
			energy_per_rotation = 20000,
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "1kW"},
			extension_speed = 0.07,
			rotation_speed = 0.04,
			fast_replaceable_group = "inserter",
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-filter-long-inserter/candle-stack-filter-long-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34},
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-filter-long-inserter/candle-stack-filter-long-inserter-hand-closed.png", priority = "extra-high", width = 24, height = 41},
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-filter-long-inserter/candle-stack-filter-long-inserter-hand-open.png", priority = "extra-high", width = 32, height = 41},
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-stack-filter-long-inserter/candle-stack-filter-long-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}},
			filter_count = 1, 
			stack = true, 
		},
		item =  {
			name = "candle-stack-filter-long-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-stack-filter-long-inserter.png", 
			order = "x-c-j", 
			place_result = "candle-stack-filter-long-inserter", 
		},
		recipe = {
			name = "candle-stack-filter-long-inserter", 
			ingredients = {{"stack-filter-inserter", 1}, {"iron-gear-wheel", 1}, {"iron-plate", 1}}, 
			result = "candle-stack-filter-long-inserter", 
			enabled = false, 
		}
	}, 
	["candle-advanced-inserter"] = {
		inserter = {
			name = "candle-advanced-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-advanced-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-advanced-inserter"}, 
			energy_per_movement = 9000, 
			energy_per_rotation = 9000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "0.4kW"}, 
			extension_speed = 0.0875, --0.07 
			rotation_speed = 0.05, --0.04 
			fast_replaceable_group = "inserter", 
			hand_size = 1.5, 
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-inserter/candle-advanced-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-inserter/candle-advanced-inserter-hand-closed.png", priority = "extra-high", width = 18, height = 41}, 
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-inserter/candle-advanced-inserter-hand-open.png", priority = "extra-high", width = 18, height = 41}, 
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-inserter/candle-advanced-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
			filter_count = 5
		},
		item =  {
			name = "candle-advanced-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-advanced-inserter.png", 
			order = "x-c-k", 
			place_result = "candle-advanced-inserter", 
		},
		recipe = {
			name = "candle-advanced-inserter", 
			ingredients = {{"candle-filter-long-inserter", 1}, {"electronic-circuit", 5}}, 
			result = "candle-advanced-inserter", 
			enabled = false, 
		}
	},
	["candle-advanced-stack-inserter"] = {
		inserter = {
			name = "candle-advanced-stack-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-advanced-stack-inserter.png", 
			minable = {hardness = 0.2, mining_time = 0.5, result = "candle-advanced-stack-inserter"}, 
			energy_per_movement = 21000, 
			energy_per_rotation = 21000, 
			energy_source = {type = "electric", usage_priority = "secondary-input", drain = "1kW"}, 
			extension_speed = 0.0875, 
			rotation_speed = 0.05, 
			fast_replaceable_group = "inserter", 
			hand_size = 1.5, 
			hand_base_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-stack-inserter/candle-advanced-stack-inserter-hand-base.png", priority = "extra-high", width = 8, height = 34}, 
			hand_closed_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-stack-inserter/candle-advanced-stack-inserter-hand-closed.png", priority = "extra-high", width = 24, height = 41}, 
			hand_open_picture = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-stack-inserter/candle-advanced-stack-inserter-hand-open.png", priority = "extra-high", width = 32, height = 41}, 
			platform_picture = {sheet = {filename = "__K&L-Inserters__/graphics/entity/candle-advanced-stack-inserter/candle-advanced-stack-inserter-platform.png", priority = "extra-high", width = 46, height = 46, shift = {0.09375, 0}}}, 
			filter_count = 5, 
			stack = true, 
		},
		item =  {
			name = "candle-advanced-stack-inserter", 
			icon = "__K&L-Inserters__/graphics/icons/candle-advanced-stack-inserter.png", 
			order = "x-c-l", 
			place_result = "candle-advanced-stack-inserter", 
		},
		recipe = {
			name = "candle-advanced-stack-inserter", 
			ingredients = {{"candle-stack-filter-long-inserter", 1}, {"electronic-circuit", 5}}, 
			result = "candle-advanced-stack-inserter", 
			enabled = false, 
		}
	}
}

for prototypeName, prototypeTables in next, candlePrototypes do 
	for prototypeTableName, prototypeTable in next, basePrototypes do
		for key, value in next, prototypeTable do
			prototypeTables[prototypeTableName][key] = value
		end
		
		data.raw[prototypeTableName][prototypeName] = util.table.deepcopy(prototypeTables[prototypeTableName])
	end
end